// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace DiscoveryProxy.Contracts
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.ServiceModel;
    using System.ServiceModel.Discovery;
    using System.Xml;
    using System.Xml.Linq;
    using Microsoft.Samples.Discovery.Contracts;

    [ServiceBehavior(
    InstanceContextMode = InstanceContextMode.Single,
    ConcurrencyMode = ConcurrencyMode.Multiple)]
    public class LegacyChatProxy : ILegacyChatProxy
    {
        public void Register(Uri chatUri, string name)
        {
            // Create endpoint metadata from the legacy API
            var metadata = new EndpointDiscoveryMetadata()
            {
                Address = new EndpointAddress(chatUri),
                ContractTypeNames = { new XmlQualifiedName("SimpleChat", "http://sample.microsoft.com/wcf4") },
                Extensions = { new XElement("root", new XElement("Name", name)) },
                ListenUris = { chatUri }
            };

            metadata.WriteLine("Adding legacy");

            // Add it to the cache
            ChatDiscoveryProxy.Cache.Add(metadata);
        }

        public void Unregister(Uri chatUri)
        {
            if (chatUri == null)
            {
                throw new ArgumentNullException("chatUri");
            }

            Console.WriteLine("Removing legacy {0}", chatUri.ToString());
            ChatDiscoveryProxy.Cache.Remove(chatUri);
        }

        public List<ChatUser> GetUsers()
        {
            List<ChatUser> users = new List<ChatUser>();
            FindCriteria criteria = new FindCriteria(typeof(ISimpleChatService));

            // Query to find the matching endpoints
            var query = from service in ChatDiscoveryProxy.Cache
                        where criteria.IsMatch(service)
                        select service;

            Console.WriteLine("GetUsers returning list to legacy chat");
            foreach (var serviceMetadata in query)
            {
                ChatUser user = new ChatUser();
                XElement peerNameElement = serviceMetadata.Extensions.Elements("Name").FirstOrDefault();
                if (peerNameElement != null)
                {
                    user.Username = peerNameElement.Value;
                }
                else
                {
                    user.Username = serviceMetadata.Address.ToString();
                }

                user.Address = serviceMetadata.Address.Uri.ToString();

                users.Add(user);
            }

            return users;
        }
    }
}