// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace DiscoveryProxy
{
    using System;
    using System.Net;
    using System.ServiceModel;
    using System.ServiceModel.Discovery;
    using DiscoveryProxyProject = DiscoveryProxy;

    public static class Program
    {
        public static void Main(string[] args)
        {
            Console.WriteLine("DiscoveryProxy Console Host");
            string hostName = "localhost";

            using (ServiceHost proxyHost = HostDiscoveryEndpoint(hostName))
            using (ServiceHost legacyHost = HostLegacyEndpoint(hostName))
            {
                Console.WriteLine("Press <Enter> to exit");
                Console.ReadLine();
                proxyHost.Close();
                legacyHost.Close();
            }
        }

        private static ServiceHost HostDiscoveryEndpoint(string hostName)
        {
            // Create a new ServiceHost with a singleton ChatDiscovery Proxy
            ServiceHost myProxyHost = new
                ServiceHost(new ChatDiscoveryProxy());

            string proxyAddress = "net.tcp://" +
                hostName + ":8001/discoveryproxy";

            // Create the discovery endpoint
            DiscoveryEndpoint discoveryEndpoint =
                new DiscoveryEndpoint(
                    new NetTcpBinding(),
                    new EndpointAddress(proxyAddress));

            discoveryEndpoint.IsSystemEndpoint = false;

            // Add UDP Annoucement endpoint
            myProxyHost.AddServiceEndpoint(new UdpAnnouncementEndpoint());

            // Add the discovery endpoint
            myProxyHost.AddServiceEndpoint(discoveryEndpoint);

            myProxyHost.Open();
            Console.WriteLine(
                "Discovery Proxy {0}",
                proxyAddress);

            return myProxyHost;
        }

        private static ServiceHost HostLegacyEndpoint(string hostName)
        {
            Uri proxyAddress = new Uri("net.tcp://" + hostName + ":8001/legacyproxy");

            ServiceHost legacyHost = new
                ServiceHost(new DiscoveryProxyProject.Contracts.LegacyChatProxy());

            legacyHost.AddServiceEndpoint(
                            typeof(DiscoveryProxyProject.Contracts.ILegacyChatProxy),
                            new NetTcpBinding(),
                            proxyAddress);

            legacyHost.Open();

            Console.WriteLine(
                "Legacy Proxy {0}",
                proxyAddress);

            return legacyHost;
        }
    }
}