' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------

<ServiceBehavior(
    InstanceContextMode:=InstanceContextMode.Single,
    ConcurrencyMode:=ConcurrencyMode.Multiple)>
Public Class LegacyChatProxy
    Implements ILegacyChatProxy

    Public Function GetUsers() As List(Of ChatUser) Implements ILegacyChatProxy.GetUsers
        Dim users As New List(Of ChatUser)()
        Dim criteria As New FindCriteria(GetType(ISimpleChatService))

        ' Query to find the matching endpoints
        Dim query = From service In ChatDiscoveryProxy.Cache
                    Where criteria.IsMatch(service)
                    Select service

        Console.WriteLine("GetUsers returning list to legacy chat")
        For Each serviceMetadata In query
            Dim user As New ChatUser()
            Dim peerNameElement = serviceMetadata.Extensions.<Name>
            If peerNameElement IsNot Nothing Then
                user.Username = peerNameElement.Value
            Else
                user.Username = serviceMetadata.Address.ToString()
            End If

            user.Address = serviceMetadata.Address.Uri.ToString()

            users.Add(user)
        Next serviceMetadata

        Return users
    End Function

    Public Sub Register(ByVal chatUri As System.Uri, ByVal name As String) Implements ILegacyChatProxy.Register
        ' Create endpoint metadata from the legacy API
        Dim metadata = New EndpointDiscoveryMetadata() With {.Address = New EndpointAddress(chatUri)}
        metadata.ContractTypeNames.Add(New XmlQualifiedName("SimpleChat", "http://sample.microsoft.com/wcf4"))
        metadata.Extensions.Add(<root><Name><%= name %></Name></root>)
        metadata.ListenUris.Add(chatUri)

        metadata.WriteLine("Adding legacy")

        ' Add it to the cache
        ChatDiscoveryProxy.Cache.Add(metadata)
    End Sub

    Public Sub Unregister(ByVal chatUri As System.Uri) Implements ILegacyChatProxy.Unregister
        If chatUri Is Nothing Then Throw New ArgumentNullException("chatUri")
        Console.WriteLine("Removing legacy {0}", chatUri.ToString())
        ChatDiscoveryProxy.Cache.Remove(chatUri)
    End Sub
End Class